(* 2D Eulerian incompressible constant density fluid *)

type hidden_type

type t =
{
  nx: int;
  ny: int;
  h: float;  (* grid spacing *)
  density: float;
  mutable u: Grid.t;
  mutable v: Grid.t;
  mutable p: Grid.t;
  occupied: Intgrid.t;  (* 0 means unoccupied, 1 means stirrer *)
  hidden: hidden_type;  (* preallocated storage etc. *)
}

(* initialize fluid at rest with given dimensions, density *)
val make: int -> int -> float -> float -> t

(* advances the fluid by the given timestep, with a random perturbation of
 * given size, assuming immersed solids don't change over the timestep *)
val advance: t -> float -> float -> unit

(* FOR DEBUGGING ONLY: fluid rendering is crude and maybe inconsistent... *)

(* show velocity field *)
val render: t -> unit

(* show pressure field *)
val render_pressure: t -> unit
