(* grid operations in service of Fluid module *)

open Bigarray

type t = (float, float64_elt, c_layout) Array2.t

(* create a grid with given dimensions, initially zero *)
val make: int -> int -> t

(* trilinear interpolation between integer indices:
 * x, y, and z must be in the range [0,maxdim-1) for the appropriate maxdim. *)
val interp: t -> float -> float -> float

(* Find the (undivided - no delta x etc.) divergence of the first grids,
 * and store it in the last. Assume first grids are appropriately staggered,
 * and the last is cell-centred. *)
val divergence: x:t -> y:t -> div:t -> unit

(* Poisson solve (homogenous Dirichlet boundaries):
 * Take the given guess in lhs and refine it to be close to the solution
 * of laplacian*lhs = rhs, using the two temporary grids (rhs will be
 * overwritten as well) and according to the given tolerance and maximum
 * number of iterations. We use the undivided 7-point stencil.
 * Assume Dirichlet boundaries. The first argument shows where immersed
 * solids are (nonzero entries). *)
val solve_poisson: Intgrid.t -> lhs:t -> rhs:t -> t -> t -> int -> float -> unit

(* Subtract (undivided) gradient of first grid to the other three grids.
 * Assume Dirichlet boundaries. The first arg shows location of solids *)
val sub_gradient: Intgrid.t -> t -> x:t -> y:t -> unit

(* replace given grid with scalar times grid *)
val rescale: t -> float -> unit

(* return norm of the grid *)
val norm: t -> float

