let perturb = ref 0.01
let numparticles = ref 8000
let gridsize = ref 25
let gridh = ref 1.0
let fdensity = ref 1.0
let screensize = ref 600


let initialize () =
  Arg.parse 
    ["-d", Arg.Float (fun d -> fdensity := d), " fluid density";
     "-h", Arg.Float (fun h -> gridh := h), " grid spacing";
     "-n", Arg.Int (fun n -> gridsize := n), " number of cells on each side"; 
     "-p", Arg.Int (fun n -> numparticles := n), " number of particles"; 
     "-r", Arg.Float (fun r -> perturb := r), " size of random perturbation"; 
     "-w", Arg.Int (fun w -> screensize := w), " window size (pixels)"]
    (fun s -> ()) "";
  Random.self_init ();
  Graphics.open_graph (" " ^ (string_of_int !screensize) ^ "x" ^
                       (string_of_int !screensize));
  Graphics.auto_synchronize false;
  let f = Fluid.make !gridsize !gridsize !gridh !fdensity in
  let particles = Array.make !numparticles (Particle.make 0.0 0.0) in
  for i = 0 to !numparticles-1 do
    particles.(i) <- Particle.make_random f
  done;
  let fscreen = float !screensize in
  let mousex,mousey = Graphics.mouse_pos () in
  let sx = !gridh/.fscreen *. float (!gridsize*mousex)
  and sy = !gridh/.fscreen *. float (!gridsize*mousey) in
  let stirrer = Stir.make sx sy in
  f, particles, stirrer


let mainloop (f,particles,stirrer) =
  let fscreen = float !screensize in
  let t = ref (Unix.gettimeofday ()) in
  while not (Graphics.key_pressed ()) do
    let newt = Unix.gettimeofday () in
    let dt = newt -. !t in
    if dt > 0.01 then begin
      t := newt;

      (* user stirring *)
      let mousex,mousey = Graphics.mouse_pos () in
      let sx = !gridh/.fscreen *. float (!gridsize*mousex)
      and sy = !gridh/.fscreen *. float (!gridsize*mousey) in
      Stir.move stirrer sx sy dt f;

      (* advance simulation *)
      Fluid.advance f dt !perturb;
      Array.iter (fun p -> Particle.advance p f dt) particles;

      (* render *)
      Graphics.clear_graph ();
      (*
      Fluid.render_pressure f;
      *)
      Graphics.set_color Graphics.black;
      Fluid.render f;
      (*
      Graphics.set_color Graphics.black;
      Array.iter (Particle.render f) particles;
      *)
      Graphics.set_color 0x580070;
      Stir.render f stirrer;
      Graphics.synchronize ();
    end
  done


let _ = Printexc.print mainloop (initialize ())

