type particle =
{
  mutable x: float;
  mutable y: float;
}

let make x y =
  {
    x = x;
    y = y;
  }

let make_random f =
  let maxx = f.Fluid.h *. float_of_int f.Fluid.nx
  and maxy = f.Fluid.h *. float_of_int f.Fluid.ny in
  {
    x = Random.float maxx;
    y = Random.float maxy;
  }

let render f p =
  let maxx = f.Fluid.h *. float_of_int f.Fluid.nx
  and maxy = f.Fluid.h *. float_of_int f.Fluid.ny in
  let x = int_of_float (p.x/.maxx *. float (Graphics.size_x ()))
  and y = int_of_float (p.y/.maxy *. float (Graphics.size_y ())) in
  Graphics.plot x y

let advance p f dt =
  let maxx = f.Fluid.h *. float_of_int f.Fluid.nx
  and maxy = f.Fluid.h *. float_of_int f.Fluid.ny in
  let gridx = p.x/.f.Fluid.h
  and gridy = p.y/.f.Fluid.h in
  let maxgridx = float_of_int f.Fluid.nx-.1.00001
  and maxgridy = float_of_int f.Fluid.ny-.1.00001 in
  let u = Grid.interp f.Fluid.u gridx (min maxgridy (max 0.0 (gridy-.0.5)))
  and v = Grid.interp f.Fluid.v (min maxgridx (max 0.0 (gridx-.0.5))) gridy in
  p.x <- p.x +. u*.dt;
  p.y <- p.y +. v*.dt;
  if p.x < 0.0 || p.x >= maxx || p.y < 0.0 || p.y >= maxy then begin
    p.x <- Random.float maxx;
    p.y <- Random.float maxy;
  end else ()

