(* stirrer *)

type t =
{
  mutable x: float;
  mutable y: float;
  mutable u: float;
  mutable v: float;
}




let make initx inity =
  {
    x = initx;
    y = inity;
    u = 0.0;
    v = 0.0;
  }


let change_occupied newval s f =
  let gridx = (s.x/.f.Fluid.h -. 0.5)
  and gridy = (s.y/.f.Fluid.h -. 0.5) in
  let i = int_of_float gridx
  and j = int_of_float gridy in
  if i >= 0 & i < f.Fluid.nx & j >= 0 & j < f.Fluid.ny then
    f.Fluid.occupied.{i,j} <- newval
  else ()


(* set at a given grid position *)
let set_cell_velocity f s x y =
  let i = int_of_float x
  and j = int_of_float y in
  if i >= 0 & i < f.Fluid.nx & j >= 0 & j < f.Fluid.ny then begin
    f.Fluid.u.{i,j} <- s.u;
    f.Fluid.u.{i+1,j} <- s.u;
    f.Fluid.v.{i,j} <- s.v;
    f.Fluid.v.{i,j+1} <- s.v;
  end else ()


(* we set the velocity in all the cells between current and new *)
let set_path_velocity f s newx newy =
  let gridx = s.x/.f.Fluid.h -. 0.5
  and gridy = s.y/.f.Fluid.h -. 0.5 in
  let dx = newx/.f.Fluid.h -. 0.5 -. gridx
  and dy = newy/.f.Fluid.h -. 0.5 -. gridy in
  (* set at the old position for sure *)
  set_cell_velocity f s gridx gridy;
  (* go through the other cells along the path *)
  if dx == 0.0 & dy == 0.0 then ()
  else if abs_float dx >= abs_float dy then begin
    let slope = dy/.dx in
    let x = ref gridx
    and y = ref gridy in
    if dx > 0.0 then begin
      while !x -. gridy < dx do
        x := !x +. 1.0;
        y := !y +. slope;
        set_cell_velocity f s !x !y;
      done
    end else begin
      while !x -. gridy > dx do
        x := !x -. 1.0;
        y := !y -. slope;
        set_cell_velocity f s !x !y;
      done
    end
  end else begin
    let slope = dx/.dy in
    let x = ref gridx
    and y = ref gridy in
    if dy > 0.0 then begin
      while !y -. gridy < dy do
        x := !x +. slope;
        y := !y +. 1.0;
        set_cell_velocity f s !x !y;
      done
    end else begin
      while !y -. gridy > dy do
        x := !x -. slope;
        y := !y -. 1.0;
        set_cell_velocity f s !x !y;
      done
    end
  end


let move s newx newy dt f =
  s.u <- (newx-.s.x)/.dt;
  s.v <- (newy-.s.y)/.dt;
  (* update the fluid *)
  change_occupied 0 s f;
  set_path_velocity f s newx newy;
  s.x <- newx;
  s.y <- newy;
  change_occupied 0 s f


let render f s =
  let maxx = f.Fluid.h *. float_of_int f.Fluid.nx
  and maxy = f.Fluid.h *. float_of_int f.Fluid.ny in
  let x = int_of_float (s.x/.maxx *. float (Graphics.size_x ()))
  and y = int_of_float (s.y/.maxy *. float (Graphics.size_y ()))
  and rx = int_of_float (0.5*.float (Graphics.size_x ())/.float f.Fluid.nx)
  and ry = int_of_float (0.5*.float (Graphics.size_y ())/.float f.Fluid.ny) in
  Graphics.fill_ellipse x y rx ry

